<?php

namespace Drupal\simple_cron;

use Drupal\Core\Config\Entity\DraggableListBuilder;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;

/**
 * Provides a listing of Cron job entities.
 */
class CronJobListBuilder extends DraggableListBuilder {

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_cron_list';
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    return [
      'title' => $this->t('Cron job'),
      'provider' => $this->t('Module'),
      'crontab' => $this->t('Crontab'),
      'single' => $this->t('Single run'),
      'last_run' => $this->t('Last run'),
      'next_run' => $this->t('Next run'),
      'status' => $this->t('Status'),
    ] + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\simple_cron\Entity\CronJobInterface $entity */
    $row['title']['#markup'] = $entity->label();
    $row['provider']['#markup'] = $entity->getProviderName();
    $row['crontab']['#markup'] = $entity->getCrontab();
    $row['single']['#markup'] = $entity->isSingle() ? $this->t('Yes') : $this->t('No');

    $row['last_run']['#markup'] = $entity->getLastRunTime()
      ? $this->dateFormatter()->format($entity->getLastRunTime()->getTimestamp())
      : $this->t('Never');

    $row['next_run']['#markup'] = $entity->status() && $entity->getNextRunTime()
      ? $this->dateFormatter()->format($entity->getNextRunTime()->getTimestamp())
      : $this->t('Disabled');

    $row['status']['#markup'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');

    if ($entity->isLocked()) {
      $row['status']['#markup'] = $this->t('Locked');
    }

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);

    if ($entity->access('run')) {
      $operations['run'] = [
        'title' => $this->t('Run'),
        'url' => $entity->toUrl('run'),
        'weight' => 20,
      ];
    }

    if ($entity->access('unlock')) {
      $operations['unlock'] = [
        'title' => $this->t('Unlock'),
        'url' => $entity->toUrl('unlock'),
        'weight' => 0,
      ];
    }

    return $operations;
  }

  /**
   * Get date formatter.
   *
   * @return \Drupal\Core\Datetime\DateFormatterInterface
   *   The date formatter.
   */
  protected function dateFormatter(): DateFormatterInterface {
    if (!isset($this->dateFormatter)) {
      $this->dateFormatter = \Drupal::service('date.formatter');
    }

    return $this->dateFormatter;
  }

}
