<?php

namespace Drupal\simple_cron\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_cron\CronJobManagerInterface;

/**
 * The settings form of the simple cron module.
 *
 * @package Drupal\simple_cron\Form
 */
class SimpleCronSettingsForm extends ConfigFormBase {

  public const SETTINGS_NAME = 'simple_cron.settings';

  /**
   * The cron job manager.
   *
   * @var \Drupal\simple_cron\CronJobManagerInterface
   */
  protected $cronJobManager;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_cron_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [static::SETTINGS_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $values = $this->config(static::SETTINGS_NAME);

    // Cron configuration.
    $form['cron'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Cron hook override'),
      '#tree' => TRUE,
    ];

    $form['cron']['override_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $values->get('cron.override_enabled'),
    ];

    // Queue configuration.
    $form['queue'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Queue workers override'),
      '#tree' => TRUE,
    ];

    $form['queue']['override_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $values->get('queue.override_enabled'),
    ];

    // Base configuration.
    $form['base'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Base configuration'),
      '#tree' => TRUE,
    ];

    $form['base']['max_execution_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum execution time'),
      '#default_value' => $values->get('base.max_execution_time'),
      '#field_suffix' => $this->t('seconds'),
      '#required' => TRUE,
      '#min' => 0,
      '#max' => 7200,
    ];

    $form['base']['lock_timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Lock timeout'),
      '#default_value' => $values->get('base.lock_timeout'),
      '#field_suffix' => $this->t('seconds'),
      '#required' => TRUE,
      '#min' => 0,
      '#max' => 7200,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $values = $form_state->cleanValues()->getValues();
    $config = $this->config(static::SETTINGS_NAME);

    foreach ($values as $key => $value) {
      $config->set($key, $value);
    }

    $config->save();

    $this->cronJobManager()->updateList();
    $form_state->setRedirect('entity.simple_cron_job.collection');
  }

  /**
   * Get cron job manager.
   *
   * @return \Drupal\simple_cron\CronJobManagerInterface
   *   The cron job manager.
   */
  protected function cronJobManager(): CronJobManagerInterface {
    if (!isset($this->cronJobManager)) {
      $this->cronJobManager = \Drupal::service('simple_cron.cron_job_manager');
    }

    return $this->cronJobManager;
  }

}
